/**

    Copyright 2007, 2009 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.api.caller;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.StringTokenizer;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.FileDataSource;

import org.apache.tika.Tika;
import org.apache.tika.exception.TikaException;
import org.apache.tika.metadata.Metadata;
import org.apache.tika.parser.ParseContext;
import org.apache.tika.parser.Parser;
import org.apache.tika.parser.microsoft.ooxml.OOXMLParser;
import org.apache.tika.sax.BodyContentHandler;
import org.eclipse.ebpm.metadata.Button;
import org.eclipse.ebpm.metadata.Field;
import org.eclipse.ebpm.metadata.SchemaDefinition;
import org.eclipse.ebpm.workflow.api.IControlAPI;
import org.eclipse.ebpm.workflow.api.IProcessEngine;
import org.eclipse.ebpm.workflow.api.IQueryAPI;
import org.eclipse.ebpm.workflow.api.Page;
import org.eclipse.ebpm.workflow.api.Process;
import org.eclipse.ebpm.workflow.api.ProcessInstance;
import org.eclipse.ebpm.workflow.api.Task;
import org.eclipse.ebpm.workflow.api.TaskInstance;
import org.eclipse.ebpm.workflow.api.TaskInstanceResponse;
import org.eclipse.ebpm.workflow.api.Variable;
import org.eclipse.ebpm.workflow.api.pdf.IPortableDocumentFormatDocument;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic.workflow.api.embedded.client.EmbeddableProcessEngine;
import org.xml.sax.SAXException;

public class APICaller {
	
	protected IProcessEngine engine = null;
	protected IControlAPI controlAPI = null;
	protected IQueryAPI queryAPI = null;
	protected String actorId = null;
	
	public static final Logger logger = LoggerFactory.getLogger(APICaller.class);
	// Id Process Instance, Process Name
	Map<Long, String> startedInstances = null;
	
	public APICaller(IProcessEngine engine){
		controlAPI = engine.getControlAPI();
		queryAPI = engine.getQueryAPI();
		
	}
	
	public void open(PrintWriter pw) throws Exception {
		LoggerFactory.getLogger(this.getClass());
		try {
			String cmd = null;
		
			Scanner in = new Scanner(System.in);

		    while (true){
		    	pw.print(" -> ");
		    	pw.flush();
		    	cmd = in.nextLine();
		    	
		    	if ((cmd.equalsIgnoreCase("stop")) || (cmd.equalsIgnoreCase("close")) || (cmd.equalsIgnoreCase("exit")))
		    		break;
		    	else
		    		try{
		    				execute(cmd, pw);
		    		}catch (Throwable e) {
		    			e.printStackTrace();
		    		}
		    }
		
		} finally{
			engine.stop();
		}
	}
	
	public void execute(String executeCmd, PrintWriter pw){
		StringTokenizer st  = new StringTokenizer(executeCmd);
		if (st.hasMoreTokens()){
			String cmd = st.nextToken().trim();
			if  (cmd.startsWith("help")){
				pw.println(" ======================================= Command List ============================================ ");
				pw.println(" def.list ");
				pw.println(" def.tasks <definition_name>");
				pw.println(" def.task <definition_id> <task-name>");
				pw.println(" def.startask <definition_name>");
				pw.println(" subdef <definition_name>");
				pw.println(" new <definition_name> ");
				pw.println(" new.att <definition_name> <att_id> <file_name>");
				pw.println(" upload <instanceID> <att_id> <file_name>");
				pw.println(" atts <instanceID>");
				pw.println(" download <instanceID> <att_id> ");
				
				pw.println(" pi.list");
				pw.println(" pi.list <definition_name?> ");
				pw.println(" pi.abort <instanceID> ");
				pw.println(" pi.set <instanceID> <varName!> <varValue!> ");
				pw.println(" pi.state <instanceID>  ");
				pw.println(" pi.byactor <actorID>  ");
				pw.println(" pi.byvarvalue <varname!> <varvalue!> ");
				pw.println(" ti <instancesId>  ");		   							
				pw.println(" ti.accept <instanceID> <taskInstanceId!> ");      		
				pw.println(" ti.complete <instanceID> <taskInstanceId!>  ");     		
				pw.println(" ti.reject <instanceID> <taskInstanceId!> ");  
				if (actorId != null){	
					pw.println(" my.tasklist  ( Accepted + Acceptable )");

					pw.println(" my.accepted ");
					pw.println(" my.acceptable ");
					pw.println(" my.acceptable_paged_ordered <pageNumber> <number_of_elements_per_page>");

					pw.println(" tasks.byprocess  <instanceID!>");
					pw.println(" getStartPDF <processName> <dest_file_name>");
					pw.println(" actor.unset");
					
				}
				else{
					pw.println(" actor.set <actorId!> ");
				}
				
				pw.println(" stop"); 
			}else if (cmd.equalsIgnoreCase("def.list")){
				Process[] defs = queryAPI.getProcesses();
				showProcessList(defs, pw);
			}else if (cmd.equalsIgnoreCase("def.task")){
				String processDefId = st.nextToken().trim();
				String taskName = st.nextToken().trim();
				Task task = queryAPI.getHumanTaskWithName(Long.valueOf(processDefId), taskName);
				Task[] taskDefs = new Task[1];
				taskDefs[0] = task;
				showTaskDefList(taskDefs,pw);
			}else if (cmd.equalsIgnoreCase("def.tasks")){
				String processName = st.nextToken().trim();
				Task[] taskDefs = queryAPI.getHumanTask(processName);
				showTaskDefList(taskDefs,pw);
			}	else if (cmd.equalsIgnoreCase("def.startask")){
				
					String par = st.nextToken().trim();
					Task startTask = null;
					try{
						long processID = Long.valueOf(par);
						startTask = queryAPI.getStartTask(processID);
					}catch (NumberFormatException nfe) {
						startTask = queryAPI.getStartTask(par);
					}
					
					Task[] taskDefs = new Task[1];
					taskDefs[0] = startTask;
					showTaskDefList(taskDefs,pw);
			}else if (cmd.equalsIgnoreCase("subdef")){
				String processName = st.nextToken().trim();
				Process[] subProcesses = queryAPI.getSubProcesses(processName);
				showProcessList(subProcesses,pw);
			}else if  (cmd.equalsIgnoreCase("new")){
				
				String processName = st.nextToken().trim();
				long instanceId = 1;
				
				if ( actorId != null ){
					instanceId = controlAPI.startByProcessNameAndActor(processName,actorId);
				}else{
					instanceId = controlAPI.startByProcessName(processName);
				}
				
				pw.println(instanceId + " Created ");
			}else if  (cmd.equalsIgnoreCase("new.att")){
				String processName = st.nextToken().trim();
				String attId = st.nextToken().trim();
				String fileName = st.nextToken().trim();
				File f = new File(fileName);
				FileDataSource fds = new FileDataSource(f);
				
				DataSource ds = new DataSourceWithMimeType(fds, mimeType(f));
				DataHandler dh = new DataHandler(ds);
				
				Map<String, DataHandler> attMap = new HashMap<String, DataHandler>();
				attMap.put(attId, dh);
				
				long instanceId = controlAPI.startByProcessName(processName, attMap);
				pw.println(instanceId + " Created ");	
			}else if  (cmd.equalsIgnoreCase("upload")){	
				Long pid = Long.valueOf(st.nextToken().trim());
				String attId = st.nextToken().trim();
				String fileName = st.nextToken().trim();
				
				
				File f = new File(fileName);
				FileDataSource fds = new FileDataSource(f);
				
				DataSource ds = new DataSourceWithMimeType(fds, mimeType(f));
				DataHandler dh = new DataHandler(ds);
				
				Map<String, DataHandler> attMap = new HashMap<String, DataHandler>();
				attMap.put(attId, dh);
				controlAPI.uploadAttachments(pid, attMap);
				
			}else if  (cmd.equalsIgnoreCase("atts")){	
				Long pid = Long.valueOf(st.nextToken().trim());
				String[] attNames = controlAPI.getAttachmentNamesForProcessInstance(pid);
				pw.println("Attachment For Process ["+pid+"]");
				for (String attN : attNames){
					pw.println("Attachment ["+attN+"]");
				}
			}else if  (cmd.equalsIgnoreCase("download")){	
				Long pid = Long.valueOf(st.nextToken().trim());
				String attId = st.nextToken().trim();
				DataHandler dh = controlAPI.downloadAttachment(pid, attId);
				
				pw.println("Attachement retrieved ["+dh.getName()+"] -> ["+dh.getContentType()+"]");
			} else if  (cmd.equalsIgnoreCase("getStartPDF")){	
				
				String processName = st.nextToken().trim();
				String destFileName = st.nextToken().trim();
				//IPortableDocumentFormatDocument pdf = queryAPI.getStartPDF(actorId, processName, "http://localhost:9092/completePDF/", "http://localhost:9092/completePDF/"); 
			
				
				IPortableDocumentFormatDocument pdf = queryAPI.getStartPDF(actorId, processName, "http://localhost:8080/PDFReceiver/ReceiverServlet", "http://localhost:8080/PDFReceiver/ReceiverServlet", null); 
				FileOutputStream fos = null;
				try{
					fos = new FileOutputStream(new File(destFileName));
					pdf.getDataHandler().writeTo(fos);
				}catch (Exception e) {
					e.printStackTrace(pw);
				}finally{
					try{
						fos.close();
					}catch (Exception e) {
						// TODO: handle exception
					}
				}
			
			}else if  (cmd.equalsIgnoreCase("pi.list")){
				String processName = null;
				ProcessInstance ins[] = null;
				if (st.hasMoreTokens()){
					processName = st.nextToken();
				}
				
				if (processName == null){
					ins = queryAPI.getProcessInstances();
				}else {
					ins = queryAPI.getProcessInstances(processName);
				}
				showProcessInstances(ins,pw);
			}else if  (cmd.equalsIgnoreCase("pi.byactor")){
				String actor = null;
				ProcessInstance ins[] = null;
				if (st.hasMoreTokens()){
					actor = st.nextToken();
				}
				
				
				ins = queryAPI.getProcessInstancesByActor(actor, false);
				
				showProcessInstances(ins,pw);
			}else if  (cmd.equalsIgnoreCase("pi.byvarvalue")){
				String varName = null;
				String varValue = null;
				ProcessInstance ins[] = null;
				if (st.hasMoreTokens()){
					varName = st.nextToken();
				}
				if (st.hasMoreTokens()){
					varValue = st.nextToken();
				}
				
				
				ins = queryAPI.getProcessInstancesByVariableValue(new Variable(varName, varValue));
				
				showProcessInstances(ins,pw);
			}else if  (cmd.equalsIgnoreCase("pi.abort")){
				String instanceID = st.nextToken();
				controlAPI.abort(Long.valueOf(instanceID));
			}else if  (cmd.equalsIgnoreCase("pi.set")){
				if (st.countTokens() < 3){
					pw.println(" The pi.set command must be in the form pi.set <INSTANCE_ID> <VAR_NAME> <VAR_VALUE>");
				}
				String instanceID = st.nextToken();
				String varName = st.nextToken();
				String varValue = st.nextToken();
				controlAPI.setGlobalVariable(Long.valueOf(instanceID), varName, varValue);
				
			}else if  (cmd.equalsIgnoreCase("pi.state")){
				if (st.countTokens() < 1){
					pw.println(" The pi.state command must be in the form pi.state <INSTANCE_ID> ");
				}
				String instanceID = st.nextToken();
				Variable[] vars = controlAPI.getGlobalVariables(Long.valueOf(instanceID));
				
				showVariables(vars,pw);
			}else if  (cmd.equalsIgnoreCase("ti")){
				if (st.countTokens() < 1){
					pw.println(" The pi.state command must be in the form pi.state <INSTANCE_ID> ");
				}
				String instanceID = st.nextToken();
				TaskInstance[] tis = queryAPI.getHumanTaskInstances(Long.valueOf(instanceID));
				
				showTaskInstances(tis,pw);
			}else if  (cmd.equalsIgnoreCase("ti.accept")){
				String instanceID = st.nextToken();
				String taskInstanceId = st.nextToken();
				
				if (actorId != null){
					controlAPI.startTaskByActor(actorId, Long.valueOf(instanceID), Long.valueOf(taskInstanceId), null);
				}else{
					controlAPI.startTask(Long.valueOf(instanceID), Long.valueOf(taskInstanceId), null);
				}
			}else if  (cmd.equalsIgnoreCase("ti.complete")){
				String instanceID = st.nextToken();
				String taskInstanceId = st.nextToken();
				
				if (actorId != null){
					controlAPI.executeTaskByActor(actorId, Long.valueOf(instanceID), Long.valueOf(taskInstanceId), null);
				}else{
					controlAPI.executeTask(Long.valueOf(instanceID), Long.valueOf(taskInstanceId), null);
				}
			}else if  (cmd.equalsIgnoreCase("ti.reject")){
				String instanceID = st.nextToken();
				String taskInstanceId = st.nextToken();
				
				if (actorId != null){
					controlAPI.rejectTaskByActor(actorId, Long.valueOf(instanceID), Long.valueOf(taskInstanceId));
				}else{
					controlAPI.rejectTask(Long.valueOf(instanceID), Long.valueOf(taskInstanceId));
				}
			}else if  (cmd.equalsIgnoreCase("my.tasklist")){
				
				
				TaskInstance[] tis = queryAPI.getHumanTaskInstancesForActor(actorId);
				showTaskInstances(tis,pw);
				 

			
			}else if(cmd.equalsIgnoreCase("tasks.byprocess")){
				// wrapper metodo di collezione dei task spettanti all'attore in base all' istanza di processo
				String instanceID = st.nextToken();
				
				Long processInstanceId = Long.parseLong(instanceID);
				
				
				//prendo il task da assegnare al valutatore per la specifica istanza
				TaskInstance[] tis = queryAPI.getHumanTaskInstancesForActorAndProcessInstance(actorId,processInstanceId);
				
				
				showTaskInstances(tis,pw);
				
			}else if  (cmd.equalsIgnoreCase("my.accepted")){
				
				TaskInstanceResponse tisr = queryAPI.getHumanTaskInstancesAssignedToActor(actorId, false);
				showAcceptedTaskInstances(tisr.getTaskInstances(),pw);
			
			}else if  (cmd.equalsIgnoreCase("my.acceptable")){
				
				TaskInstance[] tis = queryAPI.getHumanTaskInstancesForActor(actorId);
				showAcceptableTaskInstances(tis,pw);

			}else if  (cmd.equalsIgnoreCase("test1")){
				
				
				String processId = st.nextToken();
			
				Map<String, List<String>> filters = new HashMap<String, List<String>>();
				List<String> var1Values = new ArrayList<String>();
				var1Values.add("1");
				List<String> var2Values = new ArrayList<String>();
				var2Values.add("2");
				List<String> var3Values = new ArrayList<String>();
				var3Values.add("3");
				
				filters.put("A", var1Values);
				filters.put("B", var2Values);
				filters.put("C", var3Values);
				TaskInstanceResponse tisr = queryAPI.getHumanTaskInstancesForProcessAndActorFiltered(Long.valueOf(processId), "su", Page.nr(1).size(10), null, filters, false);
				showTaskInstances(tisr.getTaskInstances(), pw);
			} else if  (cmd.equalsIgnoreCase("test2")){
					
					
					String processId = st.nextToken();
				
					Map<String, List<String>> filters = new HashMap<String, List<String>>();

					List<String> var2Values = new ArrayList<String>();
					var2Values.add("2");
					List<String> var3Values = new ArrayList<String>();
					var3Values.add("3");
					

					filters.put("B", var2Values);
					filters.put("C", var3Values);
					TaskInstanceResponse tisr = queryAPI.getHumanTaskInstancesForProcessAndActorFiltered(Long.valueOf(processId), "su", Page.nr(1).size(10), null, filters, false);
					showTaskInstances(tisr.getTaskInstances(), pw);	
			}	else if  (cmd.equalsIgnoreCase("test3")){
						
						
						String processId = st.nextToken();
					
						Map<String, List<String>> filters = new HashMap<String, List<String>>();

						List<String> var1Values = new ArrayList<String>();
						var1Values.add("1");
						
						filters.put("A", var1Values);
						TaskInstanceResponse tisr = queryAPI.getHumanTaskInstancesForProcessAndActorFiltered(Long.valueOf(processId), "su", Page.nr(1).size(10), null, filters, false);
						showTaskInstances(tisr.getTaskInstances(), pw);	
			}else if  (cmd.equalsIgnoreCase("testmulti")){
							
							
							String processId = st.nextToken();
						
							Map<String, List<String>> filters = new HashMap<String, List<String>>();

							List<String> var1Values = new ArrayList<String>();
							var1Values.add("1");
							var1Values.add("2");
							
							filters.put("A", var1Values);
							TaskInstanceResponse tisr = queryAPI.getHumanTaskInstancesForProcessAndActorFiltered(Long.valueOf(processId), "su", Page.nr(1).size(10), null, filters, false);
							showTaskInstances(tisr.getTaskInstances(), pw);	
			} else if  (cmd.equalsIgnoreCase("testmulti2")){
								
								
								String processId = st.nextToken();
							
								Map<String, List<String>> filters = new HashMap<String, List<String>>();

								List<String> var1Values = new ArrayList<String>();
								var1Values.add("1");
								var1Values.add("2");
								
								List<String> var2Values = new ArrayList<String>();
								var2Values.add("2");
								
								filters.put("A", var1Values);
								filters.put("B", var2Values);
								TaskInstanceResponse tisr = queryAPI.getHumanTaskInstancesForProcessAndActorFiltered(Long.valueOf(processId), "su", Page.nr(1).size(10), null, filters, false);
								showTaskInstances(tisr.getTaskInstances(), pw);	
			}else if  (cmd.equalsIgnoreCase("actor.unset")){
				this.actorId = null;
			}else if  (cmd.equalsIgnoreCase("actor.set")){
				this.actorId = st.nextToken().trim();
			}else{
				pw.println("Comando non riconosciuto");
			}
		}
	}
	
	/*
	private List<Long> retrieveProcessEvaluate(String actorId2) {
		
		List<Long> result = new ArrayList<Long>();
		// accesso 
		SAXReader reader = new SAXReader();
		
		// salvataggio dell'istanza di processo in cui l'utente � valutatore
		try {
			Document doc = reader.read(new File("C:\\Scrappy\\valutatori.xml"));
			List<Node> nodes = doc.selectNodes("/VALUTATORI/*");
			for(Node n : nodes){
			 if(actorId2.equals(n.valueOf("@userID"))){
				Long l = Long.parseLong(n.valueOf("@processo"));
				 pw.println("Utente :" + n.valueOf("@user")+" processo in cui � valutatore "+n.valueOf("@processo"));
				 result.add(l);
			 }
			 }} catch (DocumentException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
		
		return result;
	
	}
*/
	/*
	private TaskInstance[] filtraDipartimento(TaskInstance[] tis) {
		//chiamata alla base dati di profilatura per estrapolare il dipartimento dello user
		String dep = retrieveDepartment(actorId);
		
		List<TaskInstance> result = new ArrayList<TaskInstance>(); 
		if(tis != null){
			for(TaskInstance t : tis){
				//retirve processInstanceId
				Long idp = t.getProcessInstanceId();
				//retrieve TaskInstace id
				//Long idt = t.getTaskInstanceId();
				//estrapolo il valore del dipartimento
				Variable v = controlAPI.getGlobalVariable(idp, "department");
				if(v.getValue().equals(dep))
					result.add(t);		
			}
		}
		return result.toArray(new TaskInstance[result.size()]);
	}
	*/

	public String getActorId() {
		return actorId;
	}

	public void setActorId(String actorId) {
		this.actorId = actorId;
	}

	/*
	private String retrieveDepartment(String actorId2) {
		// accesso 
		SAXReader reader = new SAXReader();
		String dep = null;
		try {
			Document doc = reader.read(new File("C:\\Scrappy\\userprofile.xml"));
			List<Node> nodes = doc.selectNodes("/AUTHORIZATIONS/ENTITIES/USERS/*");
			for(Node n : nodes){
			 if(actorId2.equals(n.valueOf("@userID"))){
				 dep = n.valueOf("@department");
				 pw.println("Utente :" + n.valueOf("@user")+" dipartimento "+n.valueOf("@department"))	;
			 }
				 
			}
		
		} catch (DocumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		
		return dep;
	}

*/
	public String printStatus(int i){
		String status = "UNKNOW";
		if (i == 0){
			status = "NOT_GENERATED";
		}
		
		if (i == 1){
			status = "GENERATED";
		}
		
		
		if (i == 2){
			status =  "COMPLETED";
		}
		
		if (i == 3){
			status =  "CANCELLED";
		}
		
		if (i == 4){
			status =  "STARTED";
		}
		
		if (i == 5){
			status =  "EXPIRED";
		}
		return status;
	}
	
	public String printStatusProcess(int i){
	
		
		
		String status = "UNKNOW";
		switch (i) {
		case ProcessInstance.INSTANCE_RUNNING:
			status = "INSTANCE_RUNNING";
			break;
		case ProcessInstance.INSTANCE_EXECUTED:
			status = "INSTANCE_EXECUTED";
			break;
		case ProcessInstance.INSTANCE_FAULTED:
			status = "INSTANCE_FAULTED";
			break;
		case ProcessInstance.INSTANCE_KILLED:
			status = "INSTANCE_KILLED";
			break;	
		case ProcessInstance.INSTANCE_EXPIRED:
			status = "INSTANCE_EXPIRED";
			break;
			
		default:
			status = "INSTANCE_EXPIRED";
			break;
		}
		return status;
		
		
		
	}
	
	
	
	public void showProcessList(org.eclipse.ebpm.workflow.api.Process[] defs, PrintWriter pw){
		pw.println("-- Process Definitions --");
		for (org.eclipse.ebpm.workflow.api.Process d : defs){
			pw.println("-- Process ID ["+d.getProcessId()+"] Name ["+d.getName()+"] Version ["+d.getVersion()+"]");
		}
	}
	public void showTaskDefList(Task[] taskDefs, PrintWriter pw){
		for ( Task task : taskDefs){
			pw.println ( " -- Task ["+task.getTaskId()+"] ["+task.getTaskName()+"]");
			SchemaDefinition sd = task.getSchemaDefinition();
			if ( sd != null ){
				pw.println ( " ---> Metadata ["+sd.getName()+"]\n");
				List<Field> flds = sd.getFields();
				for (Field aFld : flds){
					pw.println ( " ----> FLD ["+aFld.getId()+"] - ["+aFld.getName()+"]\n");
				}
				List<Button> btns = sd.getButtons();
				for (Button aButton : btns){
					pw.println ( " ----> BUTTON ["+aButton.getId()+"] - ["+aButton.getStatus()+"] ["+aButton.getAction()+"]\n");
				}
			}
			

			if (task.getEnabledRoles() != null){
				for ( String role : task.getEnabledRoles()){
					pw.println ( " ----> Role ( "+ role + " )");	
				}
			}
		}
	}	
	public void showProcessInstances(ProcessInstance[] ins, PrintWriter pw){
		for (ProcessInstance i : ins){
			pw.println("-- Process Instance ["+i.getProcessInstanceId()+"] Name ["+i.getName()+"] State ["+printStatusProcess(i.getState())+"]");
		}
	}
	
	public void showVariables(Variable[] vars,PrintWriter pw){
		for (Variable v : vars){
			pw.println("-- Variable  [ "+v.getName()+"]");
			pw.println(v.getValue());	
		}
	}
	
	public void showTaskInstances(TaskInstance[] tis,PrintWriter pw){
		for (TaskInstance ti : tis){
			pw.println("-- TI ["+ti.getTaskInstanceId()+"] [PID:"+ti.getProcessInstanceId()+"]- ["+ti.getProcessName() + "." + ti.getTaskName() +"] ("+printStatus(ti.getStatus())+") ( "+ti.getStatus()+")- OWNER ["+ti.getAssignedTo()+"]");
		}
	}
	
	public void showAcceptedTaskInstances(TaskInstance[] tis,PrintWriter pw){
		for (TaskInstance ti : tis){
			if ( ti.getAssignedTo() != null )
				pw.println("-- TI ["+ti.getTaskInstanceId()+"] - ["+ti.getProcessName() + "." + ti.getTaskName() +"] ("+printStatus(ti.getStatus())+") - OWNER ["+ti.getAssignedTo()+"] - CREATION_DATE ["+ti.getCreationDate()+"] - EXPIRATION_DATE ["+ti.getExpirationDate()+"]");
		}
	}
	
	
	public void showAcceptableTaskInstances(TaskInstance[] tis,PrintWriter pw){	
		for (TaskInstance ti : tis){
			if ( ti.getAssignedTo() == null )
				pw.println("-- TI ["+ti.getTaskInstanceId()+"] - ["+ti.getProcessName() + "." + ti.getTaskName() +"] ("+printStatus(ti.getStatus())+") - OWNER ["+ti.getAssignedTo()+"] - CREATION_DATE ["+ti.getCreationDate()+"] - EXPIRATION_DATE ["+ti.getExpirationDate()+"]");
		}
	}
	
	public String printInstanceState(int i ){
		String state = "UNKNOW";
		
		if (i == 0)
			state = "RUNNING";
		
		if (i == 1)
			state = "EXECUTED";
		
		if (i == 2)
			state = "FAULTED";
		
		if (i == 3)
			state = "KILLED";
		
		if (i == 4)
			state = "EXPIRED";
		
		return state;
	}
	
	/*
	public void status(ProcessInstance pi){
		jbpmContext.save(pi);
		pw.println(" ======================================================================================================");
		pw.println("[ Process Instance ID ] -> ["+pi.getId()+"] of Process ["+pi.getProcessDefinition().getName()+"]");
		pw.println("[ Process Instance Key ] -> ["+pi.getKey()+"]");
		pw.println("[ Process Instance Version ] -> ["+pi.getVersion()+"]");
		pw.println("[ Process Instance Started At] -> ["+pi.getStart()+"]" );
		pw.println("[ Process Instance End At] -> ["+pi.getEnd()+"]" );
		pw.println("[ Process Is Finished] -> ["+pi.hasEnded()+"]" );
		
		Token token = pi.getRootToken();
		pw.println("[ Root Token ] -> ["+token.getNode().getName()+"]["+token.isTerminatedImplicitly()+"]");
		
		Collection<TaskInstance> tasks = getTasks(pi);
		pw.println("[ Tasks ] -> ["+(tasks != null ? tasks.size() : 0)+"]");
		
		if (tasks != null){
			for (TaskInstance ti : tasks){
			
				pw.println( "["+ti.getId()+"]["+ ti.getName()+"] -> ["+(ti.getEnd() != null ? "COMPLETED" : "STARTED")+"]["+ti.getActorId()+"]");
			}
		}
		pw.println(" ======================================================================================================");
		
	}
	
	
	public Collection<TaskInstance> getTasks(ProcessInstance instance){
		return (Collection<TaskInstance>)instance.getTaskMgmtInstance().getTaskInstances();
		
	}
		*/
	public static void main(String[] args) {
		try{
			APICaller tester = new APICaller(new EmbeddableProcessEngine());
//			APICaller tester = new APICaller(new WSProcessEngine("http://localhost:9090/wfengine/"));
			tester.open(new PrintWriter(System.out));
		}catch (Exception e) {
			e.printStackTrace();
		}
	}

	
	public String mimeType(File f){
		return getFileMimeType(f);
		/*
		try{
			Magic parser = new Magic() ;
			// getMagicMatch accepts Files or byte[],
			// which is nice if you want to test streams
			MagicMatch match = parser.getMagicMatch(f, false);
			pw.println(match.getMimeType()) ;
			return match.getMimeType();
		}catch (Exception e) {
			e.printStackTrace();
			return new MimetypesFileTypeMap().getContentType(f);
		}*/
		
	}
	
	
    public static final String getFileMimeType(File file) {  
        // Default mime type to return whenever an exception occurs.  
        String mimeType = "application/octet-stream";  
        Tika tika = new Tika();  
        InputStream stream = null;  
        try {  
           mimeType = tika.detect(file);  
           // Edge case for handling Office 2007 documents.  
           if (mimeType.equals("application/x-tika-ooxml")) {  
              org.xml.sax.ContentHandler contenthandler = new BodyContentHandler();  
              Metadata metadata = new Metadata();  
              metadata.set(Metadata.RESOURCE_NAME_KEY, file.getName());  
              // Note we are using a specific Parser that can  
              // properly detect Office 2007 file types.  
              Parser parser = new OOXMLParser();  
              stream = new FileInputStream(file);  
              parser.parse(stream, contenthandler, metadata, new ParseContext());  
              mimeType = metadata.get(Metadata.CONTENT_TYPE);  
           }  
        } catch (FileNotFoundException e) {  
           logger.error("Encountered Exception while attempting to detect " +  
              "mime type for file at path : " + file.getPath() + ". " +  
              "Returning default mime type, \"" + mimeType + "\"", e);  
        } catch (IOException e) {  
            logger.error("Encountered Exception while attempting to detect " +  
               "mime type for file at path : " + file.getPath() + ". " +  
               "Returning default mime type, \"" + mimeType + "\"", e);  
        } catch (SAXException e) {  
           logger.error("Encountered Exception while attempting to detect " +  
              "mime type for file at path : " + file.getPath() + ". " +  
              "Returning default mime type, \"" + mimeType + "\"", e);  
        } catch (TikaException e) {  
           logger.error("Encountered Exception while attempting to detect " +  
              "mime type for file at path : " + file.getPath() + ". " +  
              "Returning default mime type, \"" + mimeType + "\"", e);  
        } finally {  
           try { if (stream != null) stream.close(); } catch (IOException e) {}  
        }  
        return mimeType;  
     }  
}
